from __future__ import annotations

from pathlib import Path
from typing import Tuple

import pytest
import pytest_asyncio
from aiohttp import ClientResponseError
from tenacity import RetryError

from aioqzone.api import Loginable, UpLoginManager
from aioqzone.api.h5 import QzoneH5API
from aioqzone.model import LikeData, UgcRight
from qqqr.utils.net import ClientAdapter

pytestmark = pytest.mark.asyncio(loop_scope="module")

MOOD_TEXT = "This is a curious test :D"
COMMENT_TEXT = "Here is a kind comment :)"


def select_captcha_input(prompt: str, imgs: Tuple[bytes, ...]):
    if (root := Path("data/debug")).exists():
        for i, b in enumerate(imgs, start=1):
            with open(root / f"{i}.png", "wb") as f:
                f.write(b)
    r = []
    return r


@pytest_asyncio.fixture(loop_scope="module")
async def api(client: ClientAdapter, man: Loginable, CI: bool):
    if not CI and isinstance(man, UpLoginManager):
        man.solve_select_captcha.add_impl(select_captcha_input)
    yield QzoneH5API(client, man)


async def flow_wo_check(api: QzoneH5API):
    feed_flow = await api.index()
    assert api.qzone_tokens[api.login.uin]

    hostuin = feed_flow.vFeeds[0].userinfo.uin
    profile_flow = await api.profile(hostuin)
    flow2 = await api.get_feeds(hostuin, profile_flow.feedpage.attachinfo)
    assert api.qzone_tokens[hostuin]


async def upload_photos(api: QzoneH5API):
    from aioqzone.model.api import PhotoData

    # 腾讯原创馆
    async with api.client.get(
        "https://qlogo4.store.qq.com/qzone/949589999/949589999/100?1558079493"
    ) as r:
        upp = await api.upload_pic(await r.content.read())

    prp = await api.preupload_photos([upp], upload_hd=False)
    assert prp.photos
    return prp.photos


async def qzone_workflow(api: QzoneH5API):
    await flow_wo_check(api)
    info = await upload_photos(api)

    feed = await api.publish_mood(
        MOOD_TEXT, photos=info, sync_weibo=False, ugc_right=UgcRight.self
    )
    ownuin, appid = api.login.uin, 311
    unikey = LikeData.persudo_unikey(appid, ownuin, feed.fid)

    comment = await api.add_comment(ownuin, feed.fid, appid, COMMENT_TEXT)
    await api.internal_dolike_app(appid, unikey, curkey=unikey)

    feed_flow = await api.get_active_feeds()
    feed_dict = {i.fid: i for i in feed_flow.vFeeds}
    assert feed.fid in feed_dict

    fetched_feed = feed_dict[feed.fid]
    assert fetched_feed.common.right_info.ugc_right == UgcRight.self
    assert MOOD_TEXT in fetched_feed.summary.summary
    # BUG: dolike returns `succ` but has no effect. the fetched `isliked` is False.
    # So this assertion is disabled temperorily. FIXME!
    # assert fetched_feed.like.isliked
    assert fetched_feed.comment.comments

    comment_dict = {i.commentid: i for i in fetched_feed.comment.comments}
    assert comment.commentid in comment_dict

    fetched_comment = comment_dict[comment.commentid]
    assert fetched_comment.commentLikekey == comment.commentLikekey
    assert COMMENT_TEXT in fetched_comment.content

    detail = await api.shuoshuo(feed.fid, ownuin, appid)
    assert not detail.hasmore
    assert detail.summary.summary == fetched_feed.summary.summary
    assert detail.like.isliked == fetched_feed.like.isliked
    assert detail.comment.comments
    assert comment.commentid in [i.commentid for i in detail.comment.comments]

    count1 = await api.mfeeds_get_count()
    delete = await api.delete_ugc(feed.fid, appid)
    count2 = delete.undeal_info

    assert count1 == count2


async def test_workflow(api: QzoneH5API):
    try:
        await qzone_workflow(api)
    except RetryError as e:
        assert e.last_attempt.failed
        e = e.last_attempt.exception()
        pytest.skip(f"login failed: {e}")
    except ClientResponseError as e:
        if e.status == 500:
            pytest.skip("qzone api buzy")
        raise


@pytest.mark.parametrize("size", [100, 640])
async def test_avatar(api: QzoneH5API, size):
    resp = await api.avatar(949589999, size)
    assert resp.avatar
